/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel22-tictactoe-core.
 *
 * ipspiel22-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel22-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel22-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel22.tictactoe.core.moves.impl;

import java.util.Objects;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToeFieldState;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToePlayer;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToePosition;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToeState;

/**
 * Represents a move which places a mark on a field on the board.
 * <p>
 * Note that this move is only allowed if the field is empty.
 */
final class TicTacToePlaceMarkMove extends AbstractTicTacToeMove {

    /**
     * {@code true} if a cross is to be made, {@code false} chooses a nought instead.
     */
    private final boolean cross;
    /**
     * The position of the token placed on the board.
     */
    private final TicTacToePosition tokenPosition;

    /**
     * Creates an {@link TicTacToePlaceMarkMove} object.
     *
     * @param cross         {@code true} if a cross is to be made, {@code false} chooses a nought instead.
     * @param tokenPosition The position of the field having its state changed. It must point to an empty field.
     */
    TicTacToePlaceMarkMove(final boolean cross, final TicTacToePosition tokenPosition) {
        this.cross = cross;
        this.tokenPosition = Objects.requireNonNull(tokenPosition, "tokenPosition");
    }

    /**
     * Returns {@code true} if a cross is to be made, and {@code false} if a nought is to be made instead.
     */
    boolean isCross() {
        return this.cross;
    }

    /**
     * Returns the position of the token placed on the board.
     */
    TicTacToePosition getTokenPosition() {
        return this.tokenPosition;
    }

    @Override
    public void applyTo(final TicTacToeState state, final TicTacToePlayer player) throws GameException {
        if (this.cross != player.isUsingCrosses()) {
            throw new GameException(
                    String.format(
                            "Player %s cannot make %s on field at %s.",
                            player,
                            toFieldState(this.cross),
                            this.tokenPosition));
        }

        state.getBoard().getFieldAt(this.tokenPosition).changeState(toFieldState(this.cross));
        state.moveCompleted();
    }

    @Override
    public String toString() {
        return String.format("%s on field at %s", toFieldState(this.cross), this.tokenPosition);
    }

    /**
     * Maps a cross/nought boolean to a target {@link TicTacToeFieldState}.
     *
     * @param cross {@code true} for a cross and {@code false} for a nought.
     */
    private static TicTacToeFieldState toFieldState(final boolean cross) {
        return cross ? TicTacToeFieldState.CROSS : TicTacToeFieldState.NOUGHT;
    }
}
